/*
 *
 * This file is part of Simonstrator.KOM.
 * 
 * Simonstrator.KOM is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 * 
 * PeerfactSim.KOM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with PeerfactSim.KOM.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package de.tudarmstadt.maki.simonstrator.api.component.sensor.location;

import java.util.LinkedList;


/**
 * Actuator for location information, allowing "someone" to change a host's
 * current location. While this might not be relevant on prototypical
 * deployments, it allows us to integrate movement into applications and
 * demonstrations without interfacing directly with the platform itself.
 * 
 * @author Bjoern Richerzhagen, Julian Zobel
 * @version 19.03.2018
 */
public interface LocationActuator extends LocationSensor {

	/**
	 * Allows to set the movement speed in meter per second. Should be between min
	 * and max speed, but this is not enforced.
	 */
	public void setMovementSpeed(double speed);

	/**
	 * Updates the node's current location with the new location. There is no
	 * guarantee that this operation will succeed. IF the current location changed,
	 * the {@link LocationSensor} will accurately reflect this.
	 * 
	 * @param newLocation
	 * 
	 */
	public void updateCurrentLocation(Location newLocation);

	/**
	 * A way to interact with node movement from within applications and the like.
	 * This is an optional operation - it is expected to throw an
	 * {@link UnsupportedOperationException} if it is not supported.
	 * 
	 * @param targetLocation
	 *            {@link Location3D} the node should move to.
	 */
	public void setTargetLocation(Location targetLocation) throws UnsupportedOperationException;

	/**
	 * Returns all currently known target locations. These can be used for example
	 * to update the target location.
	 * 
	 * @return
	 */
	public LinkedList<Location> getTargetLocations();

	/**
	 * Programmatically add a new waypoint. Currently not supported.
	 * 
	 * @param waypoint
	 */
	public void addTargetLocation(Location waypoint) throws UnsupportedOperationException;

	/**
	 * Programmatically remove a waypoint. Currently not supported.
	 * 
	 * @param toRemove
	 */
	default public void removeWaypoint(Location toRemove) {
		throw new UnsupportedOperationException();
	}

	/**
	 * Gets the minimum movement speed of this movable component.
	 * 
	 * @return The minimum movement speed
	 */
	public double getMinMovementSpeed();

	/**
	 * Gets the maximum movement speed of this movable component.
	 * 
	 * @return The maximum movement speed
	 */
	public double getMaxMovementSpeed();

	/**
	 * Gets the currently set movement speed for that node. Initialized with a
	 * random value between min and max. Use setMovementSpeed() to later on change
	 * the value.
	 * 
	 * @return
	 */
	public double getMovementSpeed();

	/**
	 * Retrieve the targeted {@link Location3D}, representing the location the node
	 * is currently approaching.
	 * 
	 * @return
	 */
	public Location getCurrentTargetLocation();

}
